/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.wikipedia;

import javax.annotation.Nonnull;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxon.Builder;
import it.tidalwave.bluebill.taxonomy.birds.col.CatalogueOfLifeImporter;
import it.tidalwave.bluebill.taxonomy.elmo.ElmoTaxonomyVocabulary;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import it.tidalwave.util.Initializer;
import java.net.URL;
import org.openrdf.elmo.Entity;
import org.openrdf.elmo.LiteralManager;
import org.openrdf.elmo.ResourceManager;
import org.openrdf.elmo.sesame.SesameManager;
import org.openrdf.model.Literal;
import org.openrdf.model.Resource;
import org.openrdf.model.URI;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.repository.RepositoryException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class WikipediaImporter extends CatalogueOfLifeImporter
  {
//    private int n = 0;

    @Nonnull
    protected Builder process (final @Nonnull String path,
                               final @Nonnull String scientificName,
                               @Nonnull Builder builder)
      throws Exception
      {
//        if (++n > 10) throw new CatalogueOfLifeImporter.IgnoreException();

        if (builder.getRank() != Taxon.Rank.SPECIES)
          {
            throw new IgnoreException();
          }

        final URL url = new URL(String.format("http://en.wikipedia.org/w/index.php?title=%s&printable=yes", scientificName.replace(' ', '_')));
        final WikipediaInfoExtractor textExtractor = new WikipediaInfoExtractor(scientificName);
        textExtractor.process(url);
        final String text = textExtractor.getDescription();
        builder = builder.withDisplayNames(textExtractor.getDisplayable()).
                          withInitializer(new Initializer<Taxon>()
          {
            @Nonnull
            public Taxon initialize (final @Nonnull Taxon taxon)
              {
                try
                  {
                    final SesameManager em = (SesameManager) ElmoManagerThreadLocal.get();
                    final ResourceManager<Resource> rm = em.getResourceManager();
                    final LiteralManager<URI, Literal> lm = em.getLiteralManager();
                    em.getConnection().add(rm.createResource(taxon.getLookup().lookup(Entity.class).getQName()),
                                           new URIImpl(ElmoTaxonomyVocabulary.NS_TAXO_BASE + "description"),
                                           lm.getLiteral(text));
                    return taxon;
                  }
                catch (RepositoryException e)
                  {
                    throw new RuntimeException(e);
                  }
              }
          });

        return builder;
      }
  }
